import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:minehunter/cubit/game_cubit.dart';
import 'package:minehunter/models/game/cell.dart';
import 'package:minehunter/models/game/game.dart';
import 'package:minehunter/utils/board_animate.dart';

class CellWidget extends StatelessWidget {
  const CellWidget({
    super.key,
    required this.cell,
    required this.row,
    required this.col,
  });

  final Cell cell;
  final int row;
  final int col;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (BuildContext context, GameState gameState) {
        final Game currentGame = gameState.currentGame;
        final GameCubit gameCubit = BlocProvider.of<GameCubit>(context);

        String imageAsset = getImageAssetName(
          currentGame: currentGame,
        );

        return Container(
          decoration: BoxDecoration(
            color: getBackgroundColor(currentGame),
            border: getCellBorders(
              currentGame: currentGame,
              row: row,
              col: col,
            ),
          ),
          child: GestureDetector(
            child: AnimatedSwitcher(
              duration: const Duration(milliseconds: 100),
              transitionBuilder: (Widget child, Animation<double> animation) {
                return ScaleTransition(scale: animation, child: child);
              },
              child: Image(
                image: AssetImage(imageAsset),
                fit: BoxFit.fill,
                key: ValueKey<int>(imageAsset.hashCode),
              ),
            ),
            onTap: () {
              // Set mines on board after first player try
              if (!currentGame.isBoardMined) {
                gameCubit.addMines(forbiddenRow: row, forbiddenCol: col);
              }

              if (!(currentGame.gameWin || currentGame.gameFail)) {
                if (currentGame.reportMode) {
                  gameCubit.reportCell(
                    row: row,
                    col: col,
                  );
                } else {
                  gameCubit.walkOnCell(
                    row: row,
                    col: col,
                  );
                }
                if (gameCubit.checkGameIsFinished()) {
                  gameCubit.updateReportMode(false);
                  BoardAnimate.startAnimation(
                    gameCubit: gameCubit,
                    currentGame: currentGame,
                    animationType: currentGame.gameWin ? 'win' : 'fail',
                  );
                }
              }
            },
          ),
        );
      },
    );
  }

  /*
  * Compute image asset name, from skin and cell value/state
  */
  String getImageAssetName({
    required Game currentGame,
  }) {
    final String skin = currentGame.globalSettings.skin;

    String imageAsset = 'assets/skins/${skin}_tile_unknown.png';

    bool showSolution = currentGame.gameWin || currentGame.gameFail;
    if (!showSolution) {
      // Running game
      if (cell.isExplored) {
        if (cell.isMined) {
          // Boom
          imageAsset = 'assets/skins/${skin}_tile_mine.png';
        } else {
          // Show mines count around
          imageAsset = 'assets/skins/${skin}_tile_${cell.minesCountAround}.png';
        }
      } else {
        if (cell.isMarked) {
          // Danger!
          imageAsset = 'assets/skins/${skin}_tile_flag.png';
        }
      }
    } else {
      // Finished game
      if (cell.isMined) {
        if (cell.isExploded) {
          // Mine exploded
          imageAsset = 'assets/skins/${skin}_tile_mine.png';
        } else {
          // Mine not found
          imageAsset = 'assets/skins/${skin}_tile_mine_not_found.png';
        }
      } else {
        // Show all mines counts
        imageAsset = 'assets/skins/${skin}_tile_${cell.minesCountAround}.png';
      }
    }

    return imageAsset;
  }

  // Compute cell background color, from cell state
  Color getBackgroundColor(currentGame) {
    if (currentGame.gameWin) {
      return cell.isAnimated ? Colors.green.shade400 : Colors.green.shade500;
    }
    if (currentGame.gameFail) {
      return cell.isAnimated ? Colors.pink.shade200 : Colors.pink.shade400;
    }

    return cell.isAnimated ? Colors.white : Colors.grey.shade200;
  }

  // Compute cell borders, from board size and cell state
  Border getCellBorders({
    required Game currentGame,
    required int row,
    required int col,
  }) {
    Color cellBorderColor = Colors.grey.shade500;
    double cellBorderWidth = 4;

    // Reduce cell border width on big boards
    int boardSize = currentGame.sizeHorizontal;
    if (boardSize > 8) {
      cellBorderWidth = 2;
      if (boardSize > 10) {
        cellBorderWidth = 1;
      }
    }

    if (currentGame.gameWin) {
      cellBorderColor = Colors.green.shade700;
    } else if (currentGame.gameFail) {
      cellBorderColor = Colors.pink.shade300;
    }

    Border borders = Border.all(
      color: cellBorderColor,
      width: cellBorderWidth,
    );

    return borders;
  }
}
